@@ -1,14 +1,15 @@
 from .command import BaseCommandRegistry, Command, CommandRegistry
+from .context import Context, ContextRegistry
 from .fallback import FallbackRegistry
-from .manager import CommandsManager
-from .setup import Setup, SetupRegistry
+from .manager import CommandsManager, Config
 
 __all__ = [
-    "Setup",
-    "SetupRegistry",
+    "Context",
+    "ContextRegistry",
     "Command",
     "BaseCommandRegistry",
     "CommandRegistry",
     "FallbackRegistry",
+    "Config",
     "CommandsManager",
 ]
