@@ -9,16 +9,36 @@ from typing import (
     Iterable,
     List,
     Optional,
+    Tuple,
     Union,
     overload,
 )
 
 
+def calc_status_diff(
+    before: Dict[str, bool], after: Dict[str, bool]
+) -> Dict[str, bool]:
+    """Calculate two status diff, which ones are changed.
+
+    :param before: Status before
+    :type before: Dict[str, bool]
+    :param after: Status after
+    :type after: Dict[str, bool]
+    :return: Status diff
+    :rtype: Dict[str, bool]
+    """
+    return {
+        k: v
+        for k, v in set(after.items()) - set(before.items())
+        if k in before or not v  # Same name different state or closed
+    }
+
+
 class Command:
     name: str
     keywords: Iterable[str]
     groups: Iterable[str]
-    needs: Iterable[str]
+    contexts: Iterable[str]
     parameters: Iterable[str]
 
     def __init__(
@@ -27,15 +47,38 @@ class Command:
         keywords: Iterable[str],
         groups: Iterable[str],
         parameter_ignore: Iterable[str],
-        needs_ignore: Iterable[str],
+        context_ignore: Iterable[str],
         payload_parameter: str,
     ) -> None:
+        """Create a Command
+
+        :param command_func: Command handler
+        :type command_func: Callable
+        :param keywords: Keywords for the command
+        :type keywords: Iterable[str]
+        :param groups: Groups the command belongs to
+        :type groups: Iterable[str]
+        :param parameter_ignore:
+            Ignore these parameters of command handlers
+            when constructing keyword arguments to pass.
+
+            See also :attr:`Config.command_parameter_ignore`
+        :type parameter_ignore: Iterable[str]
+        :param context_ignore:
+            Do not regard these parameters of command handlers
+            as context names.
+
+            See also :attr:`Config.command_context_ignore`
+        :type context_ignore: Iterable[str]
+        :param payload_parameter: [description]
+        :type payload_parameter: str
+        """
         self.command_func = command_func
         self.name = command_func.__name__
         self.keywords = keywords
         self.groups = groups
         self.parameters = []
-        self.needs = []
+        self.contexts = []
 
         if command_func.__doc__ is None:
             self.help = "/".join(self.keywords) + " " + command_func.__name__
@@ -43,13 +86,13 @@ class Command:
             self.help = dedent(command_func.__doc__).strip()
         self.brief_help = "- " + self.help.split("\n", 1)[0]
 
-        needs_ignore = [*needs_ignore, payload_parameter]
+        context_ignore = [*context_ignore, payload_parameter]
 
         for parameter in signature(command_func).parameters:
             if parameter not in parameter_ignore:
                 self.parameters.append(parameter)
-                if parameter not in needs_ignore:
-                    self.needs.append(parameter)
+                if parameter not in context_ignore:
+                    self.contexts.append(parameter)
 
     def __call__(self, **kwargs: Any) -> Any:
         return self.command_func(
@@ -95,56 +138,9 @@ class BaseCommandRegistry:
     def set_status(self, name: str, status: bool) -> None:
         raise NotImplementedError
 
-    @overload
-    def mark_default_closed(self, *args: Callable) -> Callable:
-        ...
-
-    @overload
-    def mark_default_closed(self, *args: str) -> None:
-        ...
-
-    def mark_default_closed(
-        self, *args: Union[str, Callable]
-    ) -> Optional[Callable]:
+    def set_default_closed(self, name: str) -> None:
         raise NotImplementedError
 
-    def resolve_command_status(self, command: Command) -> bool:
-        if not self.get_status(command.name):
-            return False
-        return all(
-            self.get_status(group_name) for group_name in command.groups
-        )
-
-    def get_commands_will_open(self, name: str) -> Iterable[Command]:
-        return [
-            command
-            for command in self._groups[name]
-            if all(
-                self.get_status(group_name)
-                for group_name in [command.name, *command.groups]
-                if group_name != name
-            )
-        ]
-
-    def get_commands_will_close(self, name: str) -> Iterable[Command]:
-        return [
-            command
-            for command in self._groups[name]
-            if self.resolve_command_status(command)
-        ]
-
-
-class CommandRegistry(BaseCommandRegistry):
-    def __init__(self):
-        super().__init__()
-        self._status = {}
-
-    def get_status(self, name: str) -> bool:
-        return self._status.get(name, True)
-
-    def set_status(self, name: str, status: bool) -> None:
-        self._status[name] = status
-
     @overload
     def mark_default_closed(self, *args: Callable) -> Callable:
         ...
@@ -157,7 +153,80 @@ class CommandRegistry(BaseCommandRegistry):
         self, *args: Union[str, Callable]
     ) -> Optional[Callable]:
         for arg in args:
-            self._status[arg.__name__ if callable(arg) else arg] = False
+            name = arg.__name__ if callable(arg) else arg
+            if self.get(name):
+                raise ValueError(
+                    f"Cannot mark {name} as default closed because "
+                    "the command already registered"
+                )
+            self.set_default_closed(name)
         if args and callable(args[0]):
             return args[0]
         return None
+
+    def resolve_command_status(self, command: Command) -> bool:
+        """Resolve command status from the command itself and its groups
+
+        :param command: The command to resolve status
+        :type command: Command
+        :return: Status, ``True`` for open and ``False`` for closed
+        :rtype: bool
+        """
+        if not self.get_status(command.name):
+            return False
+        return all(
+            self.get_status(group_name) for group_name in command.groups
+        )
+
+    def open(self, name: str) -> Iterable[Command]:
+        commands_opened = [
+            command
+            for command in self._groups[name]
+            if all(
+                self.get_status(group_name)
+                for group_name in [command.name, *command.groups]
+                if group_name != name
+            )
+        ]
+        self.set_status(name, True)
+        return commands_opened
+
+    def close(self, name: str) -> Iterable[Command]:
+        commands_closed = [
+            command
+            for command in self._groups[name]
+            if self.resolve_command_status(command)
+        ]
+        self.set_status(name, False)
+        return commands_closed
+
+    def batch_update_status(
+        self, status_diff: Dict[str, bool]
+    ) -> Tuple[List[Command], List[Command]]:
+        to_close = [k for k, v in status_diff.items() if not v]
+        to_open = [k for k, v in status_diff.items() if v]
+        commands_closed: List[Command] = []
+        commands_opened: List[Command] = []
+        for name in to_close:
+            commands_closed.extend(self.close(name))
+        for name in to_open:
+            commands_opened.extend(self.open(name))
+        return commands_closed, commands_opened
+
+
+class CommandRegistry(BaseCommandRegistry):
+    def __init__(self):
+        super().__init__()
+        self._status = {}
+
+    def get_status(self, name: str) -> bool:
+        return self._status.get(name, True)
+
+    def set_status(self, name: str, status: bool) -> None:
+        self._status[name] = status
+
+    def set_default_closed(self, name: str) -> None:
+        self._status[name] = False
+
+    def calc_status_diff(self, new_status: Dict[str, bool]) -> Dict[str, bool]:
+        return calc_status_diff(self._status, new_status)
